/*
 * The org.opensourcephysics.display package contains components for rendering
 * two-dimensional scalar and vector fields.
 * Copyright (c) 2005  H. Gould, J. Tobochnik, and W. Christian.
 */
package org.opensourcephysics.display2d;
/*----------------------------------------------------------------------------------------*
 * SurfaceVertex.java                                                                     *
 *                                                                                        *
 * Surface Plotter   version 1.10    14 Oct 1996                                          *
 *                   version 1.20     8 Nov 1996                                          *
 *                   version 1.30b1  17 May 1997                                          *
 *                                                                                        *
 * Copyright (c) 1996 Yanto Suryono. All Rights Reserved.                                 *
 *                                                                                        *
 * Permission to use, copy, and distribute this software for NON-COMMERCIAL purposes      *
 * and without fee is hereby granted, provided that this copyright notice appears in all  *
 * copies and this software is not modified in any way                                    *
 *                                                                                        *
 * Please send bug reports and/or corrections/suggestions to                              *
 * Yanto Suryono <d0771@cranesv.egg.kushiro-ct.ac.jp>                                     *
 *----------------------------------------------------------------------------------------*/
import java.awt.Point;

/**
 * The class <code>SurfaceVertex</code> represents a vertex in 3D space.
 *
 * @author  Yanto Suryono
 * @version 1.30b1, 17 May 1997
 * @since   1.10
 */

public final class SurfaceVertex {
  SurfacePlot surface;
  private Point  projection;
  private int    project_index;

  /**
   * The x coordinate
   */
  public double x;

  /**
   * The y coordinate
   */
  public double y;

  /**
   * The z coordinate
   */
  public double z;

  /**
   * The constructor of <code>SurfaceVertex</code>.
   * The x and y coordinated must be in normalized form, i.e: in the range -10 .. +10.
   *
   * @param ix the x coordinate
   * @param iy the y coordinate
   * @param iz the z coordinate
   */

  SurfaceVertex(double ix, double iy, double iz, SurfacePlot sp) {
    surface=sp;
    x = ix; y = iy; z = iz;
    project_index = sp.master_project_indexV-1;
  }

  /**
   * Determines whether this vertex is invalid, i.e has invalid coordinates value.
   *
   * @return <code>true</code> if this vertex is invalid
   */

  public final boolean isInvalid() {
    return Double.isNaN(z);
  }

  /**
   * Gets the 2D projection of the vertex.
   *
   * @return the 2D projection
   */

  public final Point projection() {
    if (project_index != surface.master_project_indexV) {
      projection = surface.projector.project(x,y,(double)((z-surface.zminV)*surface.zfactorV-10));
      project_index = surface.master_project_indexV;
    }

    return projection;
  }

  public final void project() {
      projection = surface.projector.project(x,y,(double)((z-surface.zminV)*surface.zfactorV-10));
  }

  /**
   * Transforms coordinate values to fit the scaling factor of the
   * projector. This routine is only used for transforming center of projection
   * in Surface Plotter.
   */

  public final void transform() {
    x = x / surface.projector.getXScaling();
    y = y / surface.projector.getYScaling();
    z = (double)((surface.zmaxV-surface.zminV)*(z/surface.projector.getZScaling()+10)/20 + surface.zminV);
  }

}

