/*
 * The org.opensourcephysics.display3d.core package provides the basic
 * interfaces and classes that conform a basic drawing framework for 3D scenes.
 *
 * Subpackages should implement these interfaces using any of the available
 * 3D rendering mechanisms.
 *
 * Copyright (c) 2005  The Open Source Physics project
 *                     http://www.opensourcephysics.org
 */

package org.opensourcephysics.display3d.core;

/**
 * <p>Title: DrawingPanel3D</p>
 * <p>Description: DrawingPanel3D is the basic 3D drawing panel</p>
 * @author Francisco Esquembre
 * @version March 2005
 */
public interface DrawingPanel3D extends org.opensourcephysics.display3d.core.interaction.InteractionSource {
  static public final int TARGET_PANEL = 0;
  /**
   * Getting the pointer to the real JPanel in it
   * @return JFrame
   */
  public javax.swing.JPanel getJPanel();

// ---------------------------------
// Customization of the panel
// ---------------------------------
  /**
   * Sets the preferred extrema for the panel. This resets the camera
   * of the panel to its default.
   * @param minX double
   * @param maxX double
   * @param minY double
   * @param maxY double
   * @param minZ double
   * @param maxZ double
   * @see Camera
   */
  public void setPreferredMinMax (double minX, double maxX,
                                  double minY, double maxY,
                                  double minZ, double maxZ);
  /**
   * Gets the preferred minimum in the X coordinate
   * @return double
   */
  public double getPreferredMinX ();
  /**
   * Gets the preferred maximum in the X coordinate
   * @return double
   */
  public double getPreferredMaxX ();
  /**
   * Gets the preferred minimum in the Y coordinate
   * @return double
   */
  public double getPreferredMinY ();
  /**
   * Gets the preferred maximum in the Y coordinate
   * @return double
   */
  public double getPreferredMaxY ();
  /**
   * Gets the preferred minimum in the Z coordinate
   * @return double
   */
  public double getPreferredMinZ ();
  /**
   * Gets the preferred maximum in the Z coordinate
   * @return double
   */
  public double getPreferredMaxZ ();

  /**
   * Sets the preferred min and max in each dimension so that all
   * elements currently in the panel are visible.
   */
  public void zoomToFit();

  /**
   * Whether the panel should try to keep a square aspect.
   * Default value is true.
   * @param square boolean
   */
  public void setSquareAspect (boolean square);

  /**
   * Whether the panel tries to keep a square aspect.
   * @return boolean
   */
  public boolean isSquareAspect ();

  /**
   * Provides the list of visualization hints that the panel uses
   * to display the 3D scene
   * @return VisualizationHints
   * @see VisualizationHints
   */
  public VisualizationHints getVisualizationHints ();

  /**
   * Provides the Camera object used to project the scene in 3D modes.
   * @return Camera
   * @see Camera
   */
  public Camera getCamera ();

  /**
   * Paints the panel immediately from within the calling thread.
   */
  public void render();

  /**
   * Repaints the panel using the event queue.
   */
  public void repaint();

  /**
   * Adds an Element to this DrawingPanel3D.
   * @param element Element
   * @see Element
   */
  public void addElement(Element element);

  /**
   * Removes an Element from this DrawingPanel3D
   * @param element Element
   * @see Element
   */
  public void removeElement(Element element);

  /**
   * Removes all Elements from this DrawingPanel3D
   * @see Element
   */
  public void removeAllElements();

  /**
   * Gets the (cloned) list of Elements.
   * (Should be synchronized.)
   * @return cloned list
   */
  public java.util.ArrayList getElements();

}
