/*
 * The org.opensourcephysics.display3d.core package provides the basic
 * interfaces and classes that conform a basic drawing framework for 3D scenes.
 *
 * Subpackages should implement these interfaces using any of the available
 * 3D rendering mechanisms.
 *
 * Copyright (c) 2005  The Open Source Physics project
 *                     http://www.opensourcephysics.org
 */

package org.opensourcephysics.display3d.core;

/**
 * <p>Title: ElementImage</p>
 * <p>Description: Draws a circle at its position with the given size.</p>
 * <p>Because an image is essentialy a 2D object, it doesn't behave completely
 * as a 3D object. Thus, its center will be affected by transformations
 * of the element, BUT ITS SIZE WON'T. Moreover, in 3D visualizations, the
 * maximum of sizeX and sizeY is used for its horizontal size. In all other
 * views, the corresponding size is used.</p>
 * <p>Images can be rotated.
 * @author Francisco Esquembre
 * @version March 2005
 */
public interface ElementImage extends Element {
    /**
     * Sets the image file to be displayed
     * @param text the String
     */
    public void setImageFile (String file);
    /**
     * Gets the image displayed
     */
    public String getImageFile ();

    /**
     * Sets the rotation angle for the image. Default is 0.
     * @param angle the rotation angle
     */
    public void setRotationAngle (double angle);

    /**
     * Gets the rotation angle for the image
     */
    public double getRotationAngle ();

}













