/*
 * The org.opensourcephysics.display3d.core package provides the basic
 * interfaces and classes that conform a basic drawing framework for 3D scenes.
 *
 * Subpackages should implement these interfaces using any of the available
 * 3D rendering mechanisms.
 *
 * Copyright (c) 2005  The Open Source Physics project
 *                     http://www.opensourcephysics.org
 */

package org.opensourcephysics.display3d.core;

/**
 * <p>Title: VisualizationHints</p>
 * <p>Description: Hints to a DrawingPanel3D about how it should look.
 * Hints can be ignored by the panel, depending on the implementation.</p>
 * @author Francisco Esquembre
 * @version March 2005
 */
public interface VisualizationHints {
  int DECORATION_NONE = 0;
  int DECORATION_AXES = 1;
  int DECORATION_CUBE = 2;

  int CURSOR_NONE = 0;
  int CURSOR_XYZ = 1;
  int CURSOR_CUBE = 2;
  int CURSOR_CROSSHAIR = 3;

  /**
   * Types of decoration displayed. One of the following
   * <ul>
   *   <li><b>DECORATION_NONE</b>: No decoration</li>
   *   <li><b>DECORATION_AXES</b>: Display labelled axes</li>
   *   <li><b>DECORATION_CUBE</b>: Display the bounding box</li>
   * </ul>
   * @param type the desired value
   */
  public void setDecorationType (int type);
  public int getDecorationType ();

  /**
   * The cursor type when interacting with the panel. One of the following
   * <ul>
   *   <li><b>CURSOR_NONE</b>: No cursor lines are shown.</li>
   *   <li><b>CURSOR_XYZ</b>: X,Y, and Z lines are displayed. The default.</li>
   *   <li><b>CURSOR_CUBE</b>: A cube from the origing to the point is shown.</li>
   *   <li><b>CURSOR_CROSSHAIR</b>: Lines parallel to the axes that cross at the given point are shown.</li>
   * </ul>
   * @param mode the desired value
   */
  public void setCursorType (int type);
  public int getCursorType ();

  /**
   * Whether the panel should try to remove hidden lines
   * @param remove the desired value
   */
  public void setRemoveHiddenLines (boolean remove);
  public boolean isRemoveHiddenLines ();

  /**
   * Whether the panel can draw quickly when it is dragged for a
   * new view point
   * @param allow the desired value
   */
  public void setAllowQuickRedraw (boolean allow);
  public boolean isAllowQuickRedraw ();

  /**
   * Whether the panel should display far objects darker
   * @param useIt the desired value
   */
  public void setUseColorDepth (boolean useIt);
  public boolean isUseColorDepth ();

}
