/*
 * The org.opensourcephysics.display3d.core.interaction package provides the
 * basic interfaces and classes for theinteraction with 3d elements.
 *
 * Copyright (c) 2005  The Open Source Physics project
 *                     http://www.opensourcephysics.org
 */

package org.opensourcephysics.display3d.core.interaction;


/**
 *
 * <p>Title: InteractionSource</p>
 *
 * <p>Description: This is the interface for an object that the user
 * can interact with. The object contains one or several interaction targets,
 * which are hot spots that respond to user interaction by issuing
 * an interaction event.</p>
 * <p>Classes implementing this class should document what targets
 * the contain.</p>
 * <p>Classes implementing the InteractionListener interface can register
 * to receive interaction events using the <code>addInteractionListener</code>
 * method of this interface.</p>
 *
 * <p>Copyright: Open Source Physics project</p>
 * @author Francisco Esquembre
 * @version May 2005
 * @see InteractionEvent
 * @see InteractionListener
 */
public interface InteractionSource {

  /**
   * Gives access to one of the targets of this source.
   * Sources should document the list of their available targets.
   * @param target An integer number that identifies the target in the source.
   * @return InteractionTarget
   */
  InteractionTarget getInteractionTarget (int target);

  /**
   * Adds the specified interaction listener to receive interaction events
   * to any of its targets from this source.
   * @param listener An object that implements the InteractionListener interface
   * @see InteractionListener
   */
  void addInteractionListener (InteractionListener listener);

  /**
   * Removes the specified interaction listener
   * @see InteractionListener
   */
  void removeInteractionListener (InteractionListener listener);

}

