/*
 * The org.opensourcephysics.display3d.core.interaction package provides the
 * basic interfaces and classes for theinteraction with 3d elements.
 *
 * Copyright (c) 2005  The Open Source Physics project
 *                     http://www.opensourcephysics.org
 */

package org.opensourcephysics.display3d.core.interaction;

/**
 * The basic interface for an active target in an InteractionSource
 */
public interface InteractionTarget {

    /**
     * Enables/Disables the target
     * @param value boolean
     */
    void setEnabled (boolean value);

    /**
     * Returns the enabled status of the target
     * @return boolean
     */
    boolean isEnabled ();

    /**
     * Returns the action command of this target
     * @return String
     */
    String getActionCommand ();

    /**
     * Sets the action commmand for this target
     * @param command String
     */
    void setActionCommand (String command);

    /**
     * Whether the interaction with the target affects the top-level group
     * of the element that contains it (instead of only affecting the element).
     * So, for instance, if the target allows to move the element and this
     * flag is set to <code>true</code>, trying to move the element will move
     * the whole group to which the element belongs.
     * This flag only makes sense for objects of the Element class (which
     * can belong to a Group). Default value of this flag is <code>false</code>.
     * @param value boolean
     * @see org.opensourcephysics.display3d.core.Element
     */
    void setAffectsGroup (boolean value);

    /**
     * Whether the target affects the top-level group of the element
     * @return boolean
     */
    boolean getAffectsGroup ();

}
