/*
 * The org.opensourcephysics.display3d.simple3d package implements the
 * classes of the org.opensourcephysics.display3d package using
 * the so called painter's algorithm, in which the diferent parts of the
 * 3D scene are painted from back to front, thus achieving a simple, but
 * for many cases effective, hidden-line removal.
 *
 * Copyright (c) 2005  The Open Source Physics project
 *                     http://www.opensourcephysics.org
 */

package org.opensourcephysics.display3d.simple3d;

/**
 * The simple3d implementation of InteractionTarget
 */
public class InteractionTarget implements org.opensourcephysics.display3d.core.interaction.InteractionTarget {
  private final Element element;
  private final int type;
  private boolean enabled = false;
  private boolean affectsGroup = false;
  private String command=null;

  /**
   * Constructor for the class
   * @param _element Element
   * @param _type int Either Element.TARGET_POSITION or Element.TARGET_SIZE
   */
  InteractionTarget(Element _element, int _type) { element = _element; type = _type; }

    /**
     * Returns the Element that contains the target
     * @return Element
     */
    final Element getElement () { return element; }

    /**
     * Returns the type of target
     * @return int
     */
    final int getType () { return type; }

    /**
     * Enables/Disables the target
     * @param value boolean
     */
    final public void setEnabled (boolean value) { enabled = value; }

    /**
     * Returns the enabled status of the target
     * @return boolean
     */
    final public boolean isEnabled () { return enabled; }

    /**
     * Returns the action command of this target
     * @return String
     */
    final public String getActionCommand () { return command; }

    /**
     * Sets the action commmand for this target
     * @param command String
     */
    final public void setActionCommand (String command) { this.command = command; }

    /**
     * Whether the interaction with the target affects the top-level group
     * of the element that contains it (instead of only affecting the element).
     * Default is false.
     * @param value boolean
     */
    final public void setAffectsGroup (boolean value) { affectsGroup = value; }

    /**
     * Whether the target affects the top-level group
     * @return boolean
     */
    final public boolean getAffectsGroup () { return affectsGroup; }

}
