/*
 * The org.opensourcephysics.display3d.simple3d package implements the
 * classes of the org.opensourcephysics.display3d package using
 * the so called painter's algorithm, in which the diferent parts of the
 * 3D scene are painted from back to front, thus achieving a simple, but
 * for many cases effective, hidden-line removal.
 *
 * Copyright (c) 2005  The Open Source Physics project
 *                     http://www.opensourcephysics.org
 */

package org.opensourcephysics.display3d.simple3d;

import org.opensourcephysics.controls.*;

public class VisualizationHints implements org.opensourcephysics.display3d.core.VisualizationHints {
  static final int HINT_DECORATION_TYPE     = 0;
  static final int HINT_REMOVE_HIDDEN_LINES = 1;
  static final int HINT_ALLOW_QUICK_REDRAW  = 2;
  static final int HINT_USE_COLOR_DEPTH     = 3;
  static final int HINT_CURSOR_TYPE         = 4;

  // Configuration variables
  private boolean removeHiddenLines=true, allowQuickRedraw=true, useColorDepth=true;
  private int cursorType = CURSOR_XYZ;
  private int decorationType=DECORATION_CUBE;

  // Implementation variables
  /**
   * The DrawingPanel3D to which it belongs.
   * This is needed to report to it any change that implies a call to update()
   */
  private DrawingPanel3D panel;

  /**
   * Package-private constructor
   * VisualizationHints objects are obtained from DrawingPanel3Ds using the
   * getVisualizationHints() method.
   * @param _panel DrawingPanel3D
   * @see DrawingPanel3D
   */
  VisualizationHints (DrawingPanel3D _panel) { this.panel = _panel; }

  /**
   * Only for the use of the XMLLoader for DrawingPanel3D!
   * Sets the panel of these hints
   * @param aPanel DrawingPanel3D
   */
  void setPanel (DrawingPanel3D aPanel) { this.panel = aPanel; }

  public void setCursorType (int _type) {
    this.cursorType = _type;
    panel.hintChanged(HINT_CURSOR_TYPE);
  }
  final public int getCursorType ()  { return this.cursorType; }

  public void setDecorationType (int _value) {
    this.decorationType = _value;
    panel.hintChanged(HINT_DECORATION_TYPE);
  }
  final public int getDecorationType () { return this.decorationType; }

  public void setRemoveHiddenLines (boolean _value) {
    this.removeHiddenLines = _value;
    panel.hintChanged(HINT_REMOVE_HIDDEN_LINES);
  }
  final public boolean isRemoveHiddenLines () { return this.removeHiddenLines; }

  public void setAllowQuickRedraw (boolean _value) {
    this.allowQuickRedraw = _value;
    panel.hintChanged(HINT_ALLOW_QUICK_REDRAW);
  }
  final public boolean isAllowQuickRedraw () { return this.allowQuickRedraw; }

  public void setUseColorDepth (boolean _value) {
    this.useColorDepth = _value;
    panel.hintChanged(HINT_USE_COLOR_DEPTH);
  }
  final public boolean isUseColorDepth () { return this.useColorDepth; }

// ----------------------------------------------------
// XML loader
// ----------------------------------------------------

  public static XML.ObjectLoader getLoader() {
    return new VisualizationHintsLoader();
  }

  protected static class VisualizationHintsLoader extends XMLLoader {
    public void saveObject(XMLControl control, Object obj) {
      VisualizationHints hints = (VisualizationHints) obj;
      control.setValue("decoration type", hints.decorationType);
      control.setValue("cursor type", hints.cursorType);
      control.setValue("remove hidden lines", hints.removeHiddenLines);
      control.setValue("allow quick redraw", hints.allowQuickRedraw);
      control.setValue("use color depth", hints.useColorDepth);
    }

    public Object createObject(XMLControl control) {
      return new VisualizationHints((DrawingPanel3D)null);
    }

    public Object loadObject(XMLControl control, Object obj) {
      VisualizationHints hints = (VisualizationHints) obj;
      hints.decorationType = control.getInt("decoration type");
      hints.cursorType = control.getInt("cursor type");
      hints.removeHiddenLines = control.getBoolean("remove hidden lines");
      hints.allowQuickRedraw = control.getBoolean("allow quick redraw");
      hints.useColorDepth = control.getBoolean("use color depth");
      return obj;
    }
  }

}
