/*
 * The org.opensourcephysics.display3d.simple3d package implements the
 * classes of the org.opensourcephysics.display3d package using
 * the so called painter's algorithm, in which the diferent parts of the
 * 3D scene are painted from back to front, thus achieving a simple, but
 * for many cases effective, hidden-line removal.
 *
 * Copyright (c) 2005  The Open Source Physics project
 *                     http://www.opensourcephysics.org
 */

package org.opensourcephysics.display3d.simple3d.utils;

/**
 * Some utility functions
 */
public class VectorAlgebra {

  static public double norm (double[] u) {
    return Math.sqrt(u[0]*u[0] + u[1]*u[1] + u[2]*u[2]);
  }

  static public double[] crossProduct (double[] u, double[] v) {
    return new double[] {u[1]*v[2] - u[2]*v[1], u[2]*v[0] - u[0]*v[2], u[0]*v[1] - u[1]*v[0]};
  }

  static public double[] normalize (double[] u) {
    double r = norm(u);
    return new double[] {u[0]/r, u[1]/r, u[2]/r};
  }

  static public double[] normalTo (double[] vector) {
    if      (vector[0]==0.0) return new double[] {1.0, 0.0, 0.0};
    else if (vector[1]==0.0) return new double[] {0.0, 1.0, 0.0};
    else if (vector[2]==0.0) return new double[] {0.0, 0.0, 1.0};
    else {
      double norm = Math.sqrt(vector[0]*vector[0] + vector[1]*vector[1]);
      return new double[] {-vector[1]/norm, vector[0]/norm, 0.0};
    }
  }

} // end of class
