package org.opensourcephysics.numerics;


/**
 * Transformation maps coordinates from one coordinate system to another.
 */
public interface Transformation extends Cloneable {

  /**
   * Provides a copy of this transformation.
   * This is used by an OSP 3D Element that will explicitely get a clone of
   * it whenever its setTransformation() method is invoked.
   * Thus, changing the original transformation directly
   * has no effect unless a new setTransformation is invoked.
   */
  public Object clone ();

  /**
   * Transforms a given point
   * @param point double[] the coordinates to be transformed
   * (the array's contents will be changed accordingly)
   * @return double[] the transformed vector (i.e. point)
   */
  public double[] direct (double[] point);

  /**
   * The inverse transformation (if it exists).
   * If the transformation is not invertible, then a call to this
   * method must throw a UnsupportedOperationException exception.
   * @param point double[] the coordinates to be transformed
   * (the array's contents will be changed accordingly)
   * @return double[] the transformed vector (i.e. point)
   * @throws UnsupportedOperationException If the transformation is
   * not invertible
   */
  public double[] inverse (double[] point) throws UnsupportedOperationException;

}
