/*
 * The org.opensourcephysics.tools package defines classes for managing OSP
 * applications and objects.
 */
package org.opensourcephysics.tools;

import java.lang.reflect.*;
import java.rmi.*;

import org.opensourcephysics.controls.*;
import org.opensourcephysics.display.*;

/**
 * A video capture tool.
 * This simply forwards jobs to the media tool if it is available.
 *
 * @author Douglas Brown
 * @version 1.0
 */
public class VideoCaptureTool implements Tool {

  protected Tool mediaTool;

  /**
   * Creates a new VideoCaptureTool.
   *
   * @param panel the drawing panel to be captured
   */
  public VideoCaptureTool(DrawingPanel panel) {
    try {
      Class mediaClass = Class.forName(
          "org.opensourcephysics.media.VidCapMediaTool");
      Constructor constructor = mediaClass.getConstructor(
          new Class[] {DrawingPanel.class});
      if (constructor != null) {
        mediaTool = (Tool)constructor.newInstance(new Object[] {panel});
      }
    }
    catch (Exception ex) {}
  }

  /**
   * Forwards job to mediaTool, if non-null.
   *
   * @param job the job
   * @param replyTo the tool to reply to
   * @throws RemoteException
   */
  public void send(Job job, Tool replyTo) throws RemoteException {
    if (mediaTool != null) {
      mediaTool.send(job, replyTo);
    }
    else OSPLog.warning("null media tool");
  }

  /**
   * Determines whether this tool is functional.
   *
   * @returns true if media tool is available
   */
  public boolean canCapture() {
    return mediaTool != null;
  }

}
